void drawTimeCircle(int year, float[] d_arr) {
  //draw measurable information
  stroke(255, 255);
  strokeWeight(0.5);
  noFill();
  
  float d = d_arr[year-2012];
  ellipse(0, 0, d, d);
  stroke(209, 209);
  strokeWeight(2);
  noFill();
  ellipse(0, 0, 1.04*d, 1.04*d);
  float inc = TWO_PI/12.0;
  for (int i=0; i<12; i++) {
    pushMatrix();
    line(0.48*d*cos(i*inc),0.48*d*sin(i*inc),0,0.5*d*cos(i*inc),0.5*d*sin(i*inc),0);
    popMatrix();
    
    pushMatrix();
    textSize(100);
    translate(0.5*d*cos(i*inc),0.5*d*sin(i*inc),0);
    scale(0.1);
    rotateZ(i*inc);
    textAlign(LEFT,CENTER);
    //textFont(f,3);
    fill(209);
    text(month[i],70,0,0);
    popMatrix();
  }
}


void drawRecords(ArrayList<Record> records, float[] d_arr) {
  for (int i=0; i<n_rows; i++) {
    int year = records.get(i).year;
    int month = records.get(i).month;
    int day = records.get(i).day;
    float rent_hours = records.get(i).rent_hours;
    int title_id = records.get(i).title_id;
    
    float d = d_arr[year-2012];  // diameter of the current circle
    float d2;  //diameter of the next circle
    if (year < 2021) {
      d2 = d_arr[year-2012+1];
    } else {
      d2 = d;
    }
    
    PVector startPt = cmptStartXYZ(d, year, month, day);
    //drawPt(startPt.x, startPt.y, startPt.z, title_id);
    
    // draw reading trajectory
    if (rent_hours > 24) {
      ArrayList<PVector> traj = new ArrayList<PVector>();
      traj = deriveTraj(d, d2, year, month, day, rent_hours);
      drawTraj(traj, title_id);
    }
    
  }
}


color determineColor(int title_id) {
  color c;
  //int c_len = colors.length;
  
  //if (title_id < c_len) {
  //  c = colors[title_id];
  //} else {
  //  c = colors[title_id%c_len];
  //}
  
  int c_len = trnColors.length;
  
  if (title_id < c_len) {
    c = trnColors[title_id];
  } else {
    c = trnColors[title_id%c_len];
  }
  
  return c;
}


void drawPt(float x, float y, float z, int title_id) {
  color c = determineColor(title_id);
  stroke(c);
  strokeWeight(10);
  point(x, y, z);
}

void drawTraj(ArrayList<PVector> traj, int title_id) {
  color c = determineColor(title_id);
  stroke(c);
  strokeWeight(4);
  noFill();
  beginShape();
  for (int i=0; i<traj.size(); i++) {
    vertex(traj.get(i).x, traj.get(i).y, traj.get(i).z);
  }
  endShape();
}
