void loadData() {
  table = loadTable("Magic_Checkouts_Duration_Inraw_2012-2021.csv", "header");
  
  n_rows = table.getRowCount();
  //println(n_rows);
  
  for (int i = 0; i<n_rows; i++) {
    int id = table.getInt(i, 0);
    String title = table.getString(i, 1);
    int title_id = table.getInt(i, 2);
    int year = table.getInt(i, 3);
    int month = table.getInt(i, 4);
    int day = table.getInt(i, 5);
    String duration = table.getString(i, 6);
    float rent_hours = table.getFloat(i, 7);
    records.add(new Record(id, title, title_id, year, month, day, duration, rent_hours));
  }
}


int[] annualCoutCounts(ArrayList<Record> records) {
  for (int i=0; i<records.size(); i++) {
    int yeari = records.get(i).year;
    int year_id = yeari - 2012;
    annualCouts[year_id] += 1;
  }
  
  // test
  //for (int i=0; i<10; i++) {
  //  println(annualCouts[i]);
  //}
  
  return annualCouts;
}


float[] determineDiameters(int[] annualCouts) {
  float sum = 0;
  float base_d = 300;
  
  // calculate average
  for (int i=0; i<annualCouts.length; i++) {
    sum += annualCouts[i];
  }
  float average = sum / annualCouts.length;
  
  for (int i=0; i<annualCouts.length; i++) {
    d_arr[i] = base_d * annualCouts[i] / average;
  }
  
  // test
  //for (int i=0; i<10; i++) {
  //  println(d_arr[i]);
  //}
  
  return d_arr;
}


PVector cmptStartXYZ(float d, int year, int month, int day) {
  float pt_x=0, pt_y=0, pt_z;
  float r = d/2;  // radius
  float l = day/30.0*r;
  
  int start_year = 2012;
  pt_z = -200*(year-start_year);
  
  float phi = 360 - (month-1)*30 - day;
  pt_x = l * cos(radians(phi));
  pt_y = -l * sin(radians(phi));
  
  PVector pt = new PVector(pt_x, pt_y, pt_z);
  
  return pt;
}


PVector cmptXYZ(float d, float d2, int year, int month, int day, float rented_hours) {
  float pt_x=0, pt_y=0, pt_z;
  float r = d/2;  // radius
  float r2 = d2/2;
  float l = day/30.0*r;
  float l2 = day/30.0*r2;
  float real_l = l + (l2-l) / (24*365) * rented_hours;
  
  float phi = 360 - (month-1)*30 - day;
  
  //pt_x = (l+l2)/2 * cos(radians(phi));
  //pt_y = (-l-l2)/2 * sin(radians(phi));
  pt_x = real_l * cos(radians(phi));
  pt_y = -real_l * sin(radians(phi));
  
  int start_year = 2012;
  // pt_z = -year_dZ*(year-start_year) - rented_hours/(24*365)*year_dZ;
  if (year < 2022) {
    pt_z = -year_dZ*(year-start_year) - rented_hours/(24*365)*year_dZ;
  } 
  else {
    pt_z = -year_dZ*(year-start_year);
  }
  
  PVector pt = new PVector(pt_x, pt_y, pt_z);
  
  return pt;
}


ArrayList<PVector> deriveTraj(float d, float d2, int year, int month, int day, float rent_hours) {
  ArrayList<PVector> traj = new ArrayList<PVector>();
  PVector startPt = cmptStartXYZ(d, year, month, day);
  traj.add(startPt);
  
  float rented_hours = 0;
  
  while (rent_hours > 24) {
    if (day<30) {
      day += 1;
    } else if (month < 12) {
      month += 1;
      day = 1;
    } else {
      year += 1;
      month = 1;
      day = 1;
    }
    
    rented_hours += 24;
    PVector pt = cmptXYZ(d, d2, year, month, day, rented_hours);
    traj.add(pt);
    
    rent_hours -= 24;
  }
  
  return traj;
}

color[] colors = { 
  #FF0000,
  #FFC000,
  #E0FF00,
  #7EFF00,
  #21FF00,
  #00FF41,
  #00FF9F,
  #00FDFF,
  #009FFF,
  #003DFF,
  #2100FF,
  #8300FF,
  #E500FF,
  #0052FF,
  #FF007C,
  #1000FF,
  #FFFFFF,
  #B284BE,
  #FFBF00,
  #00FFFF,
  #FF9966,
  #89CFF0,
  #A1CAF1,
  #5F9EA0,
  #A9B2C3,
  #EFBBCC,
  #BD33A4,
  #FFFF99,
  #FFEF00,
  #FF0800,
  #E4717A,
  #ACE1AF,
  #B2FFFF,
  #F7E7CE,
  #DFFF00,
  #FFA700,
  #B9D9EB,
  #F88379,
  #EEDC82,
  #FF5470
};


int c_trn = 127;  // 50% transparency
color[] trnColors = {
  color(255, 0, 0, c_trn),
  color(255, 192, 0, c_trn),
  color(224, 255, 0, c_trn),
  color(0, 255, 159, c_trn),
  color(0, 253, 255, c_trn),
  color(0, 159, 255, c_trn),
  color(0, 61, 255, c_trn),
  color(131, 0, 255, c_trn),
  color(229, 0, 255, c_trn)
};
